<?php
namespace Datamints\DatamintsPiwik\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2015 Eva Rieger <e.rieger@datamints.com>, datamints GmbH
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Piwik functions.
 *
 * @package datamints_piwik
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class PiwikRepository {

	/**
	 * Settings from TypoScript
	 *
	 * @var array
	 */
	protected $settings;

	/**
	 * Set the settings array.
	 *
	 * @param $settings
	 */
	public function setSettings($settings) {
		$this->settings = $settings;
	}

	/**
	 * Creates a Piwik event.
	 *
	 * @param string $date Date to annotate
	 * @param string $message Message
	 * @param string $type Event type
	 * @return array
	 */
	public function addEvent($date, $message, $type) {
		$data = $this->getData(array(
			'method' => 'DatamintsExternalEvents.addEvent',
			'event' => array(
				'date'		=> $date,
				'message'	=> $message,
				'type'		=> $type,
			),
		));

		return $data;
	}

    /**
     * Gets available languages from Piwik LanguageManager
     * @return array
     */
	public function getLanguages() {
        $data = $this->getData(array(
            'method' => 'LanguagesManager.getAvailableLanguages',
        ));

        return $data;
    }

	/**
	 * Get request result from Piwik as JSON.
	 *
	 * @param array $params
	 * @return array
	 */
	protected function getData($params) {
		$params['format'] = 'json';

		return json_decode($this->sendRequest($params), TRUE);
	}

	/**
	 * Call Piwik API.
	 *
	 * @param array $additionalParams
	 * @return array
	 */
	protected function sendRequest($additionalParams = array()) {
		$host = $this->settings['host'];

		// If no protocol was specified in the URL, we use 'https'.
		if (substr($host, 0, 2) == '//') {
			$host = 'https:' . $host;
		}

		if (substr($host, -1) != '/')
			$host = $host . '/';

		$idsite = $this->settings['idsite'];
		$apikey = $this->settings['apikey'];
		$params = array(
			'module' => 'API',
			'idSite' => $idsite,
			'token_auth' => $apikey,
		);
		$params = array_merge_recursive($params, $additionalParams);
		$params = $this->makeUrl($params);
		$params = implode('&', $params);

		return $this->executeHttpRequest($host, $params);
	}

	/**
	 * Execute an HTTP request for the POST values via CURL.
	 *
	 * @param string $requestUrl The URL for the HTTP request
	 * @param array $postValues POST values with key / value
	 * @return array
	 */
	protected function executeHttpRequest($requestUrl, $postValues = NULL) {
		$curl = curl_init($requestUrl);
		if (!$curl) {
			return FALSE;
		}

		curl_setopt($curl, CURLOPT_POST, TRUE);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $postValues);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($curl, CURLOPT_TIMEOUT, 30);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, TRUE);
		curl_setopt($curl, CURLOPT_HEADER, FALSE);
		curl_setopt($curl, CURLOPT_USERAGENT, ini_get('user_agent'));
		$response = curl_exec($curl);
		if (curl_error($curl)) {
			throw new \Exception(curl_error($curl));
		}
		curl_close($curl);

		return $response;
	}

	/**
	 * Create URL query string for Piwik API call.
	 *
	 * @param array $array List of parameters
	 * @param string $prefix Prefix for parameters
	 * @return string
	 */
	protected function makeUrl($array, $prefix = '') {
		$params = array();
		foreach ($array as $k => $v) {
			if ($prefix) {
				$k = $prefix . '[' . $k . ']';
			}
			if (is_array($v)) {
				$params = array_merge($params, $this->makeUrl($v, $k));
			}
			else {
				$params[] = $k . '=' . urlencode($v);
			}
		}
		return $params;
		//return implode('&', $params);
	}

}
